from .BaseDriver import BaseDriver
from ..OAuthUser import OAuthUser


class BitbucketDriver(BaseDriver):
    def get_default_scopes(self):
        # https://developer.atlassian.com/cloud/bitbucket/bitbucket-cloud-rest-api-scopes/
        return ["email"]

    def get_auth_url(self):
        return "https://bitbucket.org/site/oauth2/authorize"

    def get_token_url(self):
        return "https://bitbucket.org/site/oauth2/access_token"

    def get_user_url(self):
        return "https://api.bitbucket.org/2.0/user"

    def get_email_url(self):
        return "https://api.bitbucket.org/2.0/user/emails"

    def get_request_options(self, token):
        return {"headers": {"Authorization": f"Bearer {token}"}}

    def user(self):
        user_data, token = super().user()
        email = self.get_email_by_token(token)
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["uuid"],
                    "nickname": user_data["username"],
                    "name": user_data["display_name"],
                    "email": user_data.get("email", "") or email,
                    "avatar": user_data["links"]["avatar"]["href"],
                }
            )
        )
        return user

    def get_email_by_token(self, token):
        email = ""
        if self.has_scope("email"):
            emails_data = super().get_email_by_token(token)
            for email_data in emails_data.get("values", []):
                if (
                    email_data["is_primary"]
                    and email_data["is_confirmed"]
                    and email_data["type"] == "email"
                ):
                    email = email_data["email"]
        return email

    def user_from_token(self, token):
        user_data = super().user_from_token(token)
        # fetch email if possible
        email = self.get_email_by_token(token)
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["uuid"],
                    "nickname": user_data["username"],
                    "name": user_data["display_name"],
                    "email": user_data.get("email", "") or email,
                    "avatar": user_data["links"]["avatar"]["href"],
                }
            )
        )
        return user



import requests
import json
from urllib.parse import urlencode
from requests_oauthlib import OAuth2Session
from masonite.exceptions import RouteNotFoundException
from masonite.facades import Url


class BaseDriver:
    def __init__(self, application):
        self.application = application
        self.options = {}

        self._is_stateless = False
        self._scopes = self.get_default_scopes()
        self._data = {}

    def set_options(self, options):
        self.options = options
        return self

    def get_absolute_redirect_uri(self):
        redirect_route_or_url = self.options.get("redirect")
        if not redirect_route_or_url:
            raise Exception("You should specify a redirect route name or URI")
        # convert to route with name
        try:
            redirect_url = self.application.make("router").route(redirect_route_or_url)
        except RouteNotFoundException:
            redirect_url = redirect_route_or_url
        return Url.url(redirect_url)

    def get_client(self):
        self._scopes.sort()
        return OAuth2Session(
            client_id=self.options.get("client_id"),
            redirect_uri=self.get_absolute_redirect_uri(),
            scope=self._scopes,
        )

    def redirect(self):
        client = self.get_client()
        authorization_url, state = client.authorization_url(self.get_auth_url())
        # add optional parameters
        authorization_url += "&" + urlencode(self._data)
        # self.application.make("session").set("state", state)
        self.application.make("request").session.set("state", state)
        return self.application.make("response").redirect(location=authorization_url)

    def get_token(self):
        code = self.application.make("request").input("code")
        client = self.get_client()
        response = client.fetch_token(
            self.get_token_url(),
            client_secret=self.options.get("client_secret"),
            code=code,
        )
        token = response.get("access_token")
        return client, token

    def stateless(self):
        self._is_stateless = True
        return self

    def scopes(self, scopes_list):
        self._scopes.extend(scopes_list)
        return self

    def set_scopes(self, scopes_list):
        self._scopes = scopes_list
        return self

    def has_scope(self, scope):
        return scope in self._scopes

    def with_data(self, data):
        """Add optional parameters in the redirect request that the provider might support.
        data should be a dict that will be converted into URL GET params."""
        self._data = data
        return self

    def has_invalid_state(self):
        if self._is_stateless:
            return False
        # state = self.application.make("session").get("state")
        state = self.application.make("request").session.get("state")
        return state != self.application.make("request").input("state")

    def get_default_scopes(self):
        return []

    def reset_scopes(self):
        self._scopes = self.get_default_scopes()

    def get_auth_url(self):
        raise NotImplementedError()

    def get_token_url(self):
        raise NotImplementedError()

    def get_user_url(self):
        raise NotImplementedError()

    def get_email_url(self):
        raise NotImplementedError()

    def get_request_options(self, *args):
        raise NotImplementedError()

    def get_email_by_token(self, token):
        """E-mail is often not send directly with user info, a subsequent request needs to be
        made in order to fetch user e-mail (if scopes allow it)."""
        response = requests.get(self.get_email_url(), **self.get_request_options(token))
        email_data = json.loads(response.content.decode("utf-8"))
        return email_data

    def user(self):
        # if self.has_invalid_state():
        #     raise Exception("Invalid state")
        client, token = self.get_token()
        response = client.get(self.get_user_url())
        user_data = json.loads(response.content.decode("utf-8"))
        return user_data, token

    def user_from_token(self, token):
        response = requests.get(self.get_user_url(), **self.get_request_options(token))
        user_data = json.loads(response.content.decode("utf-8"))
        return user_data



from .BaseDriver import BaseDriver
from ..OAuthUser import OAuthUser


class GithubDriver(BaseDriver):
    def get_default_scopes(self):
        return ["user:email"]

    def get_auth_url(self):
        return "https://github.com/login/oauth/authorize"

    def get_token_url(self):
        return "https://github.com/login/oauth/access_token"

    def get_user_url(self):
        return "https://api.github.com/user"

    def get_email_url(self):
        return "https://api.github.com/user/emails"

    def get_request_options(self, token):
        # https://docs.github.com/en/rest/overview/resources-in-the-rest-api#current-version
        return {
            "headers": {
                "Authorization": f"token {token}",
                "Accept": "application/vnd.github.v3+json",
            }
        }

    def get_email_by_token(self, token):
        email = ""
        if self.has_scope("user:email"):
            emails_data = super().get_email_by_token(token)
            for email_data in emails_data:
                if email_data["primary"] and email_data["verified"]:
                    email = email_data["email"]
                    break
        return email

    def user(self):
        user_data, token = super().user()
        # fetch email if possible
        email = self.get_email_by_token(token)
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["id"],
                    "nickname": user_data["login"],
                    "name": user_data["name"],
                    "email": user_data["email"] or email,
                    "avatar": user_data["avatar_url"],
                }
            )
        )
        return user

    def user_from_token(self, token):
        user_data = super().user_from_token(token)
        # fetch email if possible
        email = self.get_email_by_token(token)
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["id"],
                    "nickname": user_data["login"],
                    "name": user_data["name"],
                    "email": user_data["email"] or email,
                    "avatar": user_data["avatar_url"],
                }
            )
        )
        return user



from .BaseDriver import BaseDriver
from ..OAuthUser import OAuthUser


class FacebookDriver(BaseDriver):
    def get_default_scopes(self):
        return ["email"]

    def get_auth_url(self):
        return "https://www.facebook.com/dialog/oauth"

    def get_token_url(self):
        return "https://graph.facebook.com/oauth/access_token"

    def get_user_url(self):
        return "https://graph.facebook.com/me?"

    def get_request_options(self, token):
        return {
            "headers": {"Authorization": f"Bearer {token}", "Accept": "application/json"},
            "query": {"prettyPrint": "false"},
        }

    def user(self):
        user_data, token = super().user()
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["sub"],
                    "nickname": user_data["nickname"],
                    "name": user_data["name"],
                    "email": user_data["email"],
                    "avatar": user_data["picture"],
                }
            )
        )
        return user

    def user_from_token(self, token):
        user_data = super().user_from_token(token)
        user = (
            OAuthUser()
            .set_token(token)
            .build(
                {
                    "id": user_data["sub"],
                    "nickname": user_data["nickname"],
                    "name": user_data["name"],
                    "email": user_data["email"],
                    "avatar": user_data["picture"],
                }
            )
        )
        return user
